/*
* file CcVobResource.java
*
* Licensed Materials - Property of IBM
* Restricted Materials of IBM
*
* com.ibm.rational.wvcm.stp.cc.CcVobResource
*
*  Copyright IBM Corporation 2004, 2009.  All Rights Reserved.
* Note to U.S. Government Users Restricted Rights:  Use, duplication or
* disclosure restricted by GSA ADP  Schedule Contract with IBM Corp.
*/

package com.ibm.rational.wvcm.stp.cc;

import static com.ibm.rational.wvcm.stpex.StpExBase.PROPERTY_NAMESPACE;

import java.util.List;

import javax.wvcm.ResourceList;
import javax.wvcm.WvcmException;
import javax.wvcm.PropertyNameList.PropertyName;

/**
 * <p>
 * Base interface containing properties and methods common to
 * all VOB-resident ClearCase resources.
 * </p>
 */
public interface CcVobResource extends CcResource {

    /** The VOB in which this VOB resource resides. */
    PropertyName<CcVob> VOB =
        new PropertyName<CcVob>(PROPERTY_NAMESPACE, "vob");

    /**
     * Get the value of this proxy's {@link #VOB} property.
     * @return This resource's VOB. Will never be null.
     * @throws WvcmException
     */
    public CcVob getVob() throws WvcmException;

    /** List of attributes attached to this resource. */
    public static final PropertyName<List<CcAttribute>> ATTRIBUTE_LIST =
        new PropertyName<List<CcAttribute>>(PROPERTY_NAMESPACE, "attribute-list");
    
    /**
     *  Get the value of this proxy's {@link #ATTRIBUTE_LIST} property.
     *  @return List of CcAttribute proxies applied to this resource.  Empty list if none.
     *  @throws WvcmException if this proxy doesn't define a value for this property.
     */
    public List<CcAttribute> getAttributeList() throws WvcmException;
    
    /** Replica in which this resource resides. */
    public static final PropertyName<CcReplica> CURRENT_REPLICA =
        new PropertyName<CcReplica>(PROPERTY_NAMESPACE, "current-replica");
    
    /**
     * Returns the value of this proxy's {@link #CURRENT_REPLICA} property.
     * 
     * @return Replica in which the resource resides as a CcReplica instance.
     * @throws WvcmException
     *             if this proxy doesn't define a value for this property.
     */
    public CcReplica getCurrentReplica() throws WvcmException;

    /** List of hyperlink resources attached to this resource. */
    public static final PropertyName<ResourceList<CcHyperlink>> HYPERLINK_LIST =
        new PropertyName<ResourceList<CcHyperlink>>(PROPERTY_NAMESPACE, "hyperlink-list");

    /**
     *  Get the value of this proxy's {@link #HYPERLINK_LIST} property.
     *  @return List of CcHyperlink proxies applied to this resource.  Empty list if none.
     *  @throws WvcmException if this proxy doesn't define a value for this property.
     */
    public ResourceList<CcHyperlink> getHyperlinkList() throws WvcmException;

    /** CcLockInfo object contains the supported lock properties. */
    public static final PropertyName<CcLockInfo> LOCK_INFO =
        new PropertyName<CcLockInfo>(PROPERTY_NAMESPACE, "lock-info");

    /**
     * Returns the value of this proxy's {@link #LOCK_INFO} property.
     * 
     * @return the CcLockInfo object for the resource.
     *         <code>null</code> if the object is not locked.
     * @throws WvcmException
     *             if this proxy doesn't define a value for this property.
     */
    public CcLockInfo getLockInfo() throws WvcmException;

    /**
     * Sets (or replaces) the lock on this proxy according to the
     * {@link #LOCK_INFO} property.
     * @param lockInfo lock information for the new lock, or <code>null</code>
     *        to unlock the proxy.
     */
    public void setLockInfo(CcLockInfo lockInfo);

    
    /**
     * Does this object have local mastership? If true, this object is mastered
     * in the same replica as its VOB.  Otherwise, it is mastered at a replica
     * remote from that of its VOB.
     */
    PropertyName<Boolean> HAS_LOCAL_MASTERSHIP =
        new PropertyName<Boolean>(PROPERTY_NAMESPACE, "has-local-mastership"); //$NON-NLS-1$

    /**
     * Get the value of this proxy's {@link #HAS_LOCAL_MASTERSHIP} property.
     * @return true if this object has local mastership, else false
     * @throws WvcmException if this proxy doesn't define a value for this property.
     */
    boolean getHasLocalMastership() throws WvcmException;

    
    /** Replica which has mastery of this resource. */
    public static final PropertyName<CcReplica> CC_MASTER_REPLICA =
        new PropertyName<CcReplica>(PROPERTY_NAMESPACE, "cc-master-replica");
    
    /**
     * Returns the value of this proxy's {@link #CC_MASTER_REPLICA} property.
     * 
     * @return Replica in which the resource is mastered as a CcReplica instance.
     *         null if resource cannot be mastered.
     * @throws WvcmException
     *             if this proxy doesn't define a value for this property.
     */
   public CcReplica getMasterReplica() throws WvcmException;
 
   /**
    * <p>
    * The permissions applied to this resource.
    * </p>
    */
   PropertyName<CcPermissions> PERMISSIONS =
       new PropertyName<CcPermissions>(PROPERTY_NAMESPACE, "cc-permissions");
   
   /**
    * Get the value of this resource's {@link #PERMISSIONS} property.
    * 
    * @return A permissions object from which specific permissions 
    * information can be extracted.
    * @throws WvcmException
    */
   CcPermissions getPermissions() throws WvcmException;
   
   /**
    * Set the value of this proxy's {@link #PERMISSIONS} property.
    * 
    * @param permissions  A permissions object.  Use the one returned
    *                     by getPermissions and modify it.
    * @throws WvcmException
    *         if this proxy doesn't define a value for this property.
    */
   void setPermissions(CcPermissions permissions) throws WvcmException;

   
}
